;
; Wireless temperature sensor
;
; author: Niclas Forslund
;
; Scans Dallas 1-wire temperature sensors DS18B20 and sends back
; results over a 433MHz transmitter using the Mandolyn protocol.
;
; Since it uses a well known protocol it will be compatible with
; the Telldus products: Tellstick DUO and Tellstick NET(?).
;
; There is no support for changing the House Code (HC) or Channel Code (CC)
; HC and CC is generated from the individual 1-wire serial number.
;
; Components:
;  - PIC12F675
;  - DS18B20 Dallas Temperature sensor(s)
;  - BC547B Transistor
;  - TX433 Velleman 433 MHz transmitter module
;  - 1.5k resistor
;  - 4.7k resistor
;  - 10k resistor
;  - LED
;  - 100n decoupling condensator
;
; Hardware description:
;      Pin 8: GND
;      Pin 7: GP0: LED voltage sensor, via 10k resistor to pin 2 and LED to GND
;      Pin 6: GP1: Transmitter data, connects to pin 2 on TX433
;      Pin 5: GP2: Transmitter pwr, connects to Base BC547B via 1.5k resistor.
;      Pin 4: GP3: Data pin on DHT22.
;      Pin 3: GP4: 1-wire bus to pin 2 on DS18B20 tempsensor, 4.7k pullup resistor to pin 2
;      Pin 2: GP5: LED and DS8B20 pwr, via 4.7k resistor to pin 3 and via 10k resistor to pin 7
;      Pin 1: Vdd
;
; GPO (analog input 0) measures the voltage over the LED as a reference voltage.
; Since the AD's voltage reference is Vdd, Vdd can be calculated by reading the
; fixed voltage on GP0 with the formula Vdd = 1.776/ANS0*256.
; Here, this raw ANS0 value is sent as "Humidity".
;
; Power supply: 3*AAA batteries, unregulated.
;
; 1-wire parasite mode is not supported.
;
; Transmitter protocol
;  Mandolyn
;
; Packet content for the Mandolyn protocol
;
;  - 4-bit preamble
;  - 4-bit House Code (Here, 6 byte 1-wire id XOR:ed is used)
;  - 2-bit Channel Code (Here, 6 byte 1-wire id XOR:ed is used)
;  - 2-bit Unknown (always b'11')
;  - 1-bit Battery status (here always b'0')
;  - 7-bit Humidity (here it represents battery status)
;  - 12-bit Temperature from 1-wire sensors
;  - 2-bit packet sequence number (0-2)
;  - 2-bit Checksum


	list            	p=12f675		; list directive to define processor
    #include            <p12f675.inc>
	errorlevel 			-302			; suppress message 302 from list file
    __CONFIG _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_ON & _INTRC_OSC_NOCLKOUT

;****************************************************************************
; Assign constants
;****************************************************************************
; system defines
SAMPLE_DELAY			equ	.5  ; Delay between readings, in 2.8-second units.
                                ; 21=>1min, 107=>5min, 214=>10min
PREAMBLE				equ .12 ; b'1100'

;BATT_VOLT_PIN			equ GPIO0
;DQ_PIN					equ GPIO1
;LED_AND_TEMP_POWER_PIN	equ GPIO2
;TX_POWER_PIN			equ GPIO3
;UNUSED1_PIN            equ GPIO4
;TX_PIN					equ GPIO5

; GPIO defines
BATT_VOLT_PIN			equ GPIO0
TX_PIN					equ GPIO1
TX_POWER_PIN			equ GPIO2
DHT_PIN 				equ GPIO3
DQ_PIN					equ GPIO4
LED_AND_TEMP_POWER_PIN	equ GPIO5



OptionBits  equ b'10001111'
 	;             ||||||||---- PS0    - Prescalar watchdog 1:128
	;             |||||||----- PS1
	;             ||||||------ PS2
	;             |||||------- PSA    - Assign prescaler to watchdog
	;             ||||-------- TOSE   - LtoH edge
	;             |||--------- TOCS   - Timer0 uses IntClk
	;             ||---------- INTEDG - falling edge RB0
	;             |----------- GPPU   - pull-ups disabled

; DS18B20 function commands
CONVERT_T				equ	0x44
SCRATCHPAD_WRITE 		equ	0x4E
SCRATCHPAD_READ			equ	0xBE
SCRATCHPAD_COPY			equ	0x48
RECALL_EE				equ	0xB8
READ_POWER_SUPPLY 		equ	0xB4

; DS18B20 ROM commands
OwSearchRom				equ	0xF0
OwReadRom				equ	0x33
OwMatchRom				equ	0x55
OwSkipRom				equ	0xCC
OwSearchAlarm			equ	0xEC

#define	BANK1			banksel 0x80	; Select Bank1
#define	BANK0			banksel 0x00	; Select Bank0


;****************************************************************************
; Reserve Space for Variables (file registers)
;****************************************************************************
    cblock 0x20
        RomBuffer:8     ; Ow.searchRom
		DhtBuffer:5		; DHT data: RH_Int | RH_Dec | T_Int | T_Dec | CHKSUM
        LastDiscrep     ; Search
        LastZero        ; Search
        BitMask         ; Search
        BitIndex        ; Search
        OwTemp          ; Search
        OwByte
        DhtByte
        BitCtr
        temp
        tmp0
        PacketSequenceNumber
        PacketCounter
		SensorCount
        counter
        TemperatureLo
        TemperatureHi
        Humidity
        CounterBitDelay
        CounterBitLongDelay
        XorParity
    endc

;--< macros >------------------------------------------------------

    ; DQ High w. strong pull-up. Only used in parasite mode
Ow.dqStrPull    macro
    bsf		GPIO, DQ_PIN					; High
    BANK1
    bcf		TRISIO, DQ_PIN					; Output
    BANK0
    endm

    ; Floating DQ High
Ow.dqHi     macro
    BANK1
    bsf		TRISIO, DQ_PIN					; Input
    BANK0
    endm

    ; Pull DQ low actively
Ow.dqLo     macro
    bcf		GPIO, DQ_PIN					; Low
    BANK1
    bcf		TRISIO, DQ_PIN					; Output
    BANK0
    endm

Dht.Hi      macro
    bsf		GPIO, DHT_PIN					; High
    endm

Dht.Lo     macro
    bcf		GPIO, DHT_PIN					; Low
    endm

    ; Delay for TIME us.
    ; TIME must be in multiples of 5.
wait    macro   time
    movlw   (time/.5)-.1
    movwf   tmp0
    call    wait5u
    endm

;writeInit   macro
;    BANK1                                   ;Bank 1
;    bsf     EECON1,WREN                     ;Enable write
;    bcf     INTCON,GIE                      ;Disable INTs
;    movlw   55h                             ;Unlock write
;    movwf   EECON2                          ;
;    movlw   AAh                             ;
;    movwf   EECON2                          ;
;    bsf     EECON1,WR                       ;Start the write
;    bsf     INTCON,GIE                      ;Enable INTS
;    endm

;****************************************************************************
; Reset Vector
;****************************************************************************
	ORG     0x000       					; processor reset vector
	movwf	OSCCAL							; Calibration value in W at reset

	call Init								; Initiate the PIC and variables
	goto MainLoop							; Start the main program

;****************************************************************************
; Main program
;****************************************************************************
MainLoop
    ; Read the analog voltage
    bsf		GPIO, LED_AND_TEMP_POWER_PIN	; Turn on power to LED and 1wire-bus
    BANK1
    bsf     ANSEL, ANS0						; Select analog ANS0
    BANK0
    bsf		ADCON0, ADON					; Turn on AD
    wait    .10             				; Wait 10us for analog to stabilize
    bsf		ADCON0, GO						; Start AD
AdWaitLoop
	btfsc   ADCON0, GO						; Wait for AD to be ready
	goto	AdWaitLoop
    bcf		ADCON0, ADON					; Turn off AD
    BANK1
    bcf		ANSEL, ANS0						; Deselect analog ANS0
    BANK0
    movf	ADRESH, w
    movwf   Humidity
    rlf     Humidity

;**************************************************************************
;  Get the temperature and humidity from the DHT22 sensor
;**************************************************************************
;    call    Dht.inquire

;**************************************************************************
;  Issue a "convert" temperature command to ALL sensors to save time
;**************************************************************************
convert
    call    Ow.reset        ; reset OW devices

    movlw   OwSkipRom       ; the "skip rom" command
    call    Ow.rwByte       ;

    movlw   CONVERT_T       ; send "convert" command
    call    Ow.rwByte

    ; use 60 usec "read slots" to wait for conversion complete
rdslot
    wait    .56             ; for 60 usec 'read slot'
    call    Ow.rdBit        ; C = 1, conversion complete?
    bnc     rdslot          ; no, branch (wait)

scan
    clrf    SensorCount
    clrf    LastDiscrep                 ; start a new search for 1-wire devices
scanagn
	bsf		GPIO, LED_AND_TEMP_POWER_PIN	; Turn on power to LED and 1wire-bus
    call    OwSearchNext                    ; collect a ROM ID
    movf    RomBuffer,w
    xorlw   0x28                            ; a DS18B20 device (0x28)?
    skpz                                    ; yes, skip, else
    xorlw   0x1D^0x28                       ; a DS2423b device (0x1D)?
    skpz                                    ; yes, skip, else
    goto    chklast                         ; branch (not a DS18B20 or DS2423b)

;**************************************************************************
;  Address 1w-device and send a scratchpad_read command
;**************************************************************************
    call    Ow.reset        ; reset OW devices

    movlw   OwMatchRom      ; the "match rom" command
    call    Ow.rwByte       ;

    movlw   .8              ; Send the 64-bit rom-id
    movwf   counter
    movlw   RomBuffer
	movwf	FSR
nxt movf    INDF,w
    call    Ow.rwByte
    incf    FSR,f
    decfsz  counter,f
    goto    nxt

    movlw   SCRATCHPAD_READ ; the "read scratch" command
    call    Ow.rwByte

;**************************************************************************
; Read the Scratchpad and at the same time,
; convert Temperature to Mandolyn protocol format: (temp*16+50*16).
; Since the format received from DS18B20 is already in the right format,
; I can skip the multiplication by 16 and only add 800(320h).
;**************************************************************************
    call    Ow.rdByte
    movf    OwByte, w
    addlw   0x20
	btfsc	STATUS, C
    incf	TemperatureHi, f
	movwf	TemperatureLo

    call    Ow.rdByte
    movf    OwByte, w
    addlw   0x3
    addwf   TemperatureHi, f
    call    Ow.reset

	bcf		GPIO, LED_AND_TEMP_POWER_PIN   ; Turn off power to LED and 1wire-bus

;**************************************************************************
; Transmit all temperatures over TX433 one by one
;**************************************************************************
    bsf		GPIO, TX_POWER_PIN			; Turn on power to transmitter TX433
    movlw	.3
    movwf	PacketCounter
    clrf	PacketSequenceNumber
	clrwdt                       		; Clear the watchdog
TxLoop
    call    TxPacket
    incf	PacketSequenceNumber, f
    call    BitLongDelay				; 72ms pause between each package
    decfsz	PacketCounter, f
    goto	TxLoop
    bcf		GPIO, TX_POWER_PIN			; Turn off power to transmitter

    incf	SensorCount, f
    call    BitDelay                    ; 1 ms pause between each Sender
    clrf    TemperatureLo
    clrf    TemperatureHi

chklast
    movf    LastDiscrep,F               ; last device (Z = 1)?
    skpz                                ; yes, skip, else
    goto    scanagn                     ; branch and scan again
	bcf		GPIO, LED_AND_TEMP_POWER_PIN   ; Turn off power to LED and 1wire-bus


	; Sleep
	clrwdt              		; Clear the watchdog
	movlw	SAMPLE_DELAY
	movwf	temp
SleepLoop
	sleep
	nop
	decfsz	temp, f
	goto	SleepLoop

	goto	MainLoop            ; Start next round...



;****************************************************************************
; Init
;****************************************************************************
Init
	clrf	PacketSequenceNumber
    clrf    SensorCount

	BANK1
	clrf	VRCON			; VREN=0, VRR=0, VR3=0, VR2=0, VR1=0, VR0=0
							; (power off the comparator voltage, saves power)

;****************************************************************************
; Recommended sequence to change prescaler from TMR0 -> WDT
; without causing accidental RESET
; http://ww1.microchip.com/downloads/en/DeviceDoc/31011a.pdf
;****************************************************************************
    BANK0
    clrf    TMR0                ;Clear TMR0 and prescaler
    BANK1
    movlw   OptionBits | .7     ;Select WDT, do not change prescale value
    movwf   OPTION_REG      ;
    clrwdt                  ;Clears WDT and prescaler
    movlw   OptionBits
    movwf   OPTION_REG
    clrwdt
;****************************************************************************

	movlw	0x51				; Fosc/16, only AN0 is analog
	movwf	ANSEL

	BANK0
	bcf		ADCON0, ADFM		; AD left justified. 8-bit resolution (ADRESH)
	bcf		ADCON0, VCFG		; Use Vdd as a reference

	movlw	0x07				; Comparator Off
	movwf	CMCON				; for lowest Power consumption

	bcf		GPIO, BATT_VOLT_PIN
	bcf		GPIO, TX_PIN
	bcf		GPIO, TX_POWER_PIN
	bsf		GPIO, DHT_PIN
	bsf		GPIO, DQ_PIN
	bcf		GPIO, LED_AND_TEMP_POWER_PIN
	BANK1
	bsf		TRISIO, BATT_VOLT_PIN			; Input
	bcf		TRISIO, TX_PIN					; Output
	bcf		TRISIO, TX_POWER_PIN			; Output
	bcf		TRISIO, DHT_PIN                 ; Output
	bcf		TRISIO, DQ_PIN					; Output
	bcf		TRISIO, LED_AND_TEMP_POWER_PIN	; Output
	BANK0

	return


;****************************************************************************
; RF433 Tx functions
;****************************************************************************

TxPacket
    bcf		GPIO, TX_PIN                    ; Set TX-pin to a known state

	movlw	PREAMBLE						; 4-bit Preamble 0011
	movwf	temp
    swapf   temp,f                          ; waste 4 first bits
	call 	TxContent4bit

											; Start of chksum calculation.
    movlw   b'000'                          ; Preamble not included in chksum.
	;         |||--------- Stores XOR result of all EVEN bits
	;         ||---------- Stores XOR result of all ODD bits
	;         |----------- Last sent bit ODD/EVEN bit indicator. 1=Odd, 0=Even
    movwf   XorParity

	movlw   .6              ; XOR 6 bytes of 1wire serial number and use the
    movwf   counter			; 4 bits as House Code and 2 bits as Channel Code
    movlw   RomBuffer+.1	; from the resulting XOR:ed bytes.
	movwf	FSR
	movlw   0x00            ; Ensure a known state to start with
nhc xorwf	INDF,w
    incf    FSR,f
    decfsz  counter,f
    goto    nhc
	movwf	temp
	call	TxContent6bit	; Send both generated House Code (4-bits)
							; and generated Channel Code (2-bits)

	movlw	0xC0            				; 2-bit Constant
	movwf	temp
	call	TxContent2bit

    clrf	temp                            ; 1-bit Battery status
	call	TxContentBit                    ; Here it is not used

;	================================================
;	The next byte is usually used for Humidity value.
;	Here I use it to present the current battery status
;	since I do not have a humidity sensor.
;	This data slot can be used to whatever data.
;	================================================

    movf    Humidity,w
	movwf	temp
	call	TxContent7bit

;	================================================

	movf	TemperatureHi, w 				; 12-bit temperature total
	movwf	temp
	swapf   temp,f                          ; waste 4 first bits
	call	TxContent4bit
	movf	TemperatureLo, w
	movwf	temp
	call	TxContent8bit

	movf	PacketSequenceNumber, w			; 2-bit packet sequence number
	movwf	temp
    swapf   temp,f                          ; waste 6 first bits
    rlf     temp,f
    rlf     temp,f
	call	TxContent2bit

    movf    XorParity, w                    ; 2-bit parity xor
	movwf	temp
    swapf   temp,f                          ; waste 6 first bits
    rlf     temp,f
    rlf     temp,f
	call	TxContent2bit

    call    TxStop                          ; 1-bit Stop bit 600us
    bcf		GPIO, TX_PIN
	return


	; Send out one bit of content, left-shifted out of 'temp'
TxContentBit
	rlf		temp, f
	btfsc	STATUS, C
	goto	TxOne
	goto	TxZero

	; Send out multiple bits of content from 'temp'
TxContent8bit
	call	TxContentBit
TxContent7bit
	call	TxContentBit
TxContent6bit
	call	TxContentBit
TxContent5bit
	call	TxContentBit
TxContent4bit
	call	TxContentBit
TxContent3bit
	call	TxContentBit
TxContent2bit
	call	TxContentBit
	call	TxContentBit
	return

	; Send a '0' bit
TxZero
    movlw   b'100'           ; Odd/Even indicator bit bitflip is done.
    xorwf   XorParity, f     ; As a zero is being sent it does not affect parity
	btfss	GPIO, TX_PIN
    goto    ZL2H
    goto    ZH2L
ZL2H
	bsf		GPIO, TX_PIN
	call 	BitDelay
	call 	BitDelay
	return
ZH2L
	bcf		GPIO, TX_PIN
	call 	BitDelay
	call 	BitDelay
	return

	; Send a '1' bit
TxOne
    movlw   b'101'              ; Assume bit 3 is set -> prev. odd -> b'101'
    btfss   XorParity, 2        ; Is it an even number? if not -> 0x6
    movlw   b'110'
    xorwf   XorParity, f
	btfss   GPIO, TX_PIN
    goto    OL2H
    goto    OH2L
OL2H
    bsf		GPIO, TX_PIN
	call 	BitDelay
	bcf		GPIO, TX_PIN
	call 	BitDelay
	return
OH2L
    bcf		GPIO, TX_PIN
	call 	BitDelay
	bsf		GPIO, TX_PIN
	call 	BitDelay
    return

	; Send a Stop-bit. The stop-bit has a total bit length of 1.2ms
TxStop
	btfss   GPIO, TX_PIN
    goto    SL2H
    goto    SH2L
SL2H
    bsf		GPIO, TX_PIN
	call 	BitShortDelay
	bcf		GPIO, TX_PIN
	call 	BitShortDelay
	return
SH2L
    bcf		GPIO, TX_PIN
	call 	BitShortDelay
    bsf		GPIO, TX_PIN
	call 	BitShortDelay
    return

BitDelay
	movlw	.217                    ; 217 -> 1ms half wave pulse
	movwf	CounterBitDelay         ; with a total 72.1ms package length
	goto    $+1
	decfsz	CounterBitDelay, f
	goto	$-2
	return

BitLongDelay						; Used to pause between data transmissions
	movlw	.72                     ; 72ms measured on the real ESIC
	movwf	CounterBitLongDelay
    call    BitDelay
	decfsz	CounterBitLongDelay, f
	goto	$-2
	return

BitShortDelay						; Used when sending the stop-bit.
    movlw   .150                    ; 600us
	movwf	CounterBitDelay
	nop
	decfsz	CounterBitDelay, f
	goto	$-2
	return


;**************************************************************************
; One wire Search Rom
;**************************************************************************
OwSearch
    clrf    LastDiscrep     ; init Last Discrepency var'
OwSearchNext
    clrf    BitIndex        ; BitIndex = 0
    clrf    LastZero        ; LastZero = 0
    movlw   b'00000001'     ;
    movwf   BitMask         ; setup rom array bit mask
    call    Ow.reset        ; reset all 1-wire devices
    movlw   OwSearchRom
    call    Ow.rwByte       ; send "search rom" command
    movlw   RomBuffer       ;
    movwf   FSR             ; FSR = &RomBuffer[0]
GetBits
    call    Ow.rdBit        ; read bit A (in Carry)
    rlf     OwTemp,F        ; F = -------A
    call    Ow.rdBit        ; read bit B (compliment)
    rlf     OwTemp,W        ; W = ------AB
    rrf     OwTemp,F        ; put 'A' bit back into Carry
    andlw   0x03            ; AB = 00, a discrepency?
    bnz     SendRomBit      ; no, branch, use 'A' bit, else
Discrepency
    movf    BitIndex,W      ; 0..63
    subwf   LastDiscrep,W   ; 0..63
    bnc     IndexGTLD       ; index > last discrepency, use '0'
    bz      SendRomBit      ; index = last discrepency, use '1'
IndexLTLD                       ; index < last discrepency, use ROM
    movf    BitMask,W       ; get ROM bit
    andwf   INDF,W          ; is ROM bit '0'?
    bnz     SendRomBit      ; no, branch with C = 1, else
    clrc                    ; set C = 0 and
IndexGTLD
    movf    BitIndex,W      ; update 'LastZero'
    movwf   LastZero        ;
SendRomBit
    movf    BitMask,W       ; set ROM bit to value in Carry
    iorwf   INDF,F          ; set ROM bit (unconditionally)
    skpc                    ; is bit a '1'? yes, skip, else
    xorwf   INDF,F          ; clr ROM bit
    call    Ow.rwBit        ; send direction/qualifier bit
    rlf     BitMask,W       ;
    rlf     BitMask,F       ; increment rom array bit mask
    skpnc                   ; byte boundary? no, skip, else
    incf    FSR,F           ; bump RomBuffer array address
    incf    BitIndex,F      ; bump bit index, 0..63
    btfss   BitIndex,6      ; BitIndex = 64?  yes, skip, else
    goto    GetBits         ; do next bit
    movf    LastZero,W      ; update last discrepency
    movwf   LastDiscrep     ; a 0 value indicates end-of-list
    return

;**************************************************************************
;  use Ow.cntDs18x20 to return number of DS18B20 and DS18S20 devices
;  found on the OW bus in the 'count_b' and 'count_s' variables. 
;**************************************************************************
;Ow.cntDs18x20
;        clrf    count_s         ; clear DS18S20 counter
;        clrf    count_b         ; clear DS18B20 counter
;        clrf    LastDiscrep     ; start a new search
;next
;        call    OwSearchNext    ; collect a ROM ID
;        movf    RomBuffer+0,W   ; the "Family ID" byte
;        xorlw   0x28            ; a DS18B20 device (0x28)?
;        skpnz                   ; no, skip, else
;        incf    count_b,F       ; bump DS18B20 count
;        xorlw   0x10^0x28       ; a DS18S20 device (0x10)?
;        skpnz                   ; no, skip, else
;        incf    count_s,F       ; bump DS18S20 count
;        movf    LastDiscrep,F   ; last device (Z=1)?
;        skpz                    ; yes, skip, else
;        goto    next            ; branch (do another)
;        return                  ;
		
;**************************************************************************
; Ow.rdByte, result in 'OwByte'.
; Ow.rwByte, send byte in WREG.
;**************************************************************************
Ow.rdByte
    movlw   .8
    movwf   BitCtr
rdloop
	call	Ow.rdBit
    rrf     OwByte, f
    decfsz  BitCtr,F        ; Last bit?
    goto    rdloop          ; no, loop, else
    return                  ; exit


Ow.rwByte
    movwf   OwByte
    movlw   .8
    movwf   BitCtr
	rrf		OwByte, f
rwloop
	call	Ow.rwBit
	rrf		OwByte, f
    decfsz  BitCtr,F        ; Last bit?
    goto    rwloop          ; no, loop, else
    return                  ; exit


;**************************************************************************
;  Ow.rdBit, read bit from 1wire-bus and return result in Carry.
;  Ow.rwBit, send bit in Carry.
;**************************************************************************
Ow.rdBit
    clrc
	Ow.dqLo
	Ow.dqHi
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
    btfsc   GPIO,DQ_PIN     ; sample owpin at exactly 14 us
    setc                    ; clear Carry if '0'
	wait    .60
    return

Ow.rwBit
	btfsc	STATUS, C
	goto	OnewireSlotWrite1
	goto	OnewireSlotWrite0

OnewireSlotWrite0
	Ow.dqLo
	wait    .60
    Ow.dqHi
	return

OnewireSlotWrite1
	Ow.dqLo
	Ow.dqHi
	wait    .60
    return


;***************************************************************************
;  Dht.inquire: Read 6 byte from DHT-sensor and return result in DhtBuffer.
;***************************************************************************
Dht.inquire
    clrc
	Dht.Lo
	wait	.510
	Dht.Hi
	BANK1
    bsf		TRISIO,DHT_PIN		; Set port to Input
	BANK0
	movlw   .5                  ; Read 5 bytes from DHT22
    movwf   counter
    movlw   DhtBuffer           ;
    movwf   FSR                 ; FSR = &DhtBuffer[0]
    wait    .43                 ; Should be 43, but wait-input is only x*5
	btfsc   GPIO, DHT_PIN		; Detect DHT22. If not return to scan 1wire-bus
    goto    convert
	wait	.120
	btfsc   GPIO, DHT_PIN		; Wait for data ready to read
	goto	$-1
NextDhtByte
    movlw   .8
    movwf   BitCtr
DhtTxPrep
	btfss   GPIO, DHT_PIN       ; Wait data ready to read
	goto	DhtTxPrep
	wait	.32					; At this line we are at 50us end
	btfsc   GPIO, DHT_PIN		; DHT high, set (C)arry, else
    setc
	rlf		INDF,f
	incf	FSR,f
    decfsz  BitCtr,f            ; Last bit?
	goto	DhtTxPrep
    decfsz  counter,f           ; Last byte?
	goto	NextDhtByte

	BANK1
    bcf		TRISIO,DHT_PIN		; Set port to Output
	BANK0
	Dht.Hi

    movf    DhtBuffer+0,w
    movwf   Humidity
    rlf     Humidity

	return


;**************************************************************************
;  Ow.reset returns 1 in Carry if a presence detect pulse is found
;**************************************************************************	
Ow.reset
    clrc
    Ow.dqLo             
    wait    .500        ; Set DQ low for 500 us
    Ow.dqHi             ; Set DQ high again
    wait    .70         ; and wait 70us
    btfsc   GPIO,DQ_PIN ; sample dq_pin
    setc                ; Set Carry if '1'
    wait    .430        ; delay 430us
    return


;This takes 5us to complete
wait5u
    goto    $+1
    decfsz  tmp0,f
    goto    wait5u
    movlw   .0
    return


;noSensors
;    bcf	GPIO, LED_AND_TEMP_POWER_PIN	; Turn off power to LED and 1-wire
;
;    clrwdt                 ; Clear the watchdog
;    clrf	TMR0			; Clear the prescaler
;    sleep					; Sleep while the sensor does it's work
;    nop
;
;    bsf	GPIO, LED_AND_TEMP_POWER_PIN	; Turn on power to LED and 1-wire
;
;    clrwdt                 ; Clear the watchdog
;    clrf	TMR0			; Clear the prescaler
;    sleep					; Sleep while the sensor does it's work
;    nop
;
;    call    Ow.reset       ; reset OW devices
;    btfss   STATUS,C
;    goto    noSensors
;    return

    end